var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import express from 'express';
import { GraphQLClient } from 'graphql-request';
function getVariables(vars) {
    try {
        const variables = JSON.parse(vars);
        return variables;
    }
    catch (error) {
        return {};
    }
}
function buildHeaders(req, name, type, config) {
    var _a;
    if (config === null || config === void 0 ? void 0 : config.buildHeaders) {
        return config.buildHeaders(req, name, type);
    }
    if ((_a = config === null || config === void 0 ? void 0 : config.fetchOptions) === null || _a === void 0 ? void 0 : _a.headers) {
        return config.fetchOptions.headers;
    }
    return {};
}
export default function createServerMiddleware(graphqlServer, queries, mutations, config) {
    const app = express();
    app.use(express.json());
    const client = new GraphQLClient(graphqlServer);
    if (config === null || config === void 0 ? void 0 : config.middleware) {
        app.use(config.middleware);
    }
    function query(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const name = req.query.name;
            if (!name || !queries.has(name)) {
                res.status(404).send();
                return;
            }
            try {
                const headers = buildHeaders(req, name, 'query', config);
                const variables = getVariables(req.query.variables);
                const query = queries.get(name);
                const response = yield client.rawRequest(query, variables, headers);
                if (config === null || config === void 0 ? void 0 : config.onQueryResponse) {
                    return config.onQueryResponse(response, req, res);
                }
                return res.json(response.data);
            }
            catch (e) {
                if (config === null || config === void 0 ? void 0 : config.onQueryError) {
                    return config.onQueryError(e, req, res);
                }
                return res.status(500).send();
            }
        });
    }
    function mutate(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const name = req.query.name;
            if (!name || !mutations.has(name)) {
                res.status(404).send();
                return;
            }
            const mutation = mutations.get(name);
            try {
                const headers = buildHeaders(req, name, 'mutation', config);
                const response = yield client.request(mutation, req.body, headers);
                if (config === null || config === void 0 ? void 0 : config.onMutationResponse) {
                    return config.onMutationResponse(response, req, res);
                }
                return res.json(response);
            }
            catch (error) {
                if (config === null || config === void 0 ? void 0 : config.onMutationError) {
                    return config.onMutationError(error, req, res);
                }
                return res.status(500).send();
            }
        });
    }
    app.get('/query', query);
    app.post('/mutate', mutate);
    return app;
}
